#include <iostream>

enum TokenType {
	TT_NUM,
	TT_ADD,
	TT_MUL,
	TT_END
};

struct Token {
	size_t pos;
	TokenType type;
	int n;
	Token() {}
	Token(size_t pos, TokenType type, int n):pos(pos), type(type), n(n) {}
	Token(size_t pos, TokenType type):pos(pos), type(type) {}
};

struct InterpreterError {
	std::string message;
	size_t pos;
	InterpreterError(std::string message, size_t pos):message(message), pos(pos) {}
};

class Interpreter {
public:
	Interpreter(std::string s):m_s(s), m_pos(0) {
	}

	int interpret() {
		// Get lead token
		parse_token();

		int n = parse_add();

		if (m_token.type != TT_END) {
			throw InterpreterError("Expecting end", m_token.pos);
		}

		return n;
	}

private:
	void parse_token() {
		// skip spaces before each token
		while (m_pos < m_s.length() && isspace(m_s[m_pos])) {
			m_pos ++;
		}

		size_t token_pos = m_pos;

		if (m_pos == m_s.length()) {
			m_token = Token(m_pos, TT_END);
		}
		else if (isdigit(m_s[m_pos])) {
			int n = 0;
			do {
				n = n*10 + (m_s[m_pos] - '0');
				m_pos++;
			} while (isdigit(m_s[m_pos]));
			m_token = Token(token_pos, TT_NUM, n);
		}
		else if (m_s[m_pos] == '+') {
			m_pos++;
			m_token = Token(token_pos, TT_ADD);
		}
		else if (m_s[m_pos] == '*') {
			m_pos++;
			m_token = Token(token_pos, TT_MUL);
		}
		else {
			throw InterpreterError("Unexpected character", token_pos);
		}
	}

	int parse_num() {
		if (m_token.type != TT_NUM) {
			throw InterpreterError("Expecting number", m_token.pos);
		}

		int n = m_token.n;
		parse_token();
		return n;
	}

	int parse_mul() {
		int n = parse_num();

		while (m_token.type == TT_MUL) {
			parse_token();
			n *= parse_num();
		}

		return n;
	}

	int parse_add() {
		int n = parse_mul();

		while (m_token.type == TT_ADD) {
			parse_token();
			n += parse_mul();
		}

		return n;
	}

	std::string m_s;
	size_t m_pos;
	Token m_token;
};

int main(int argc, char *argv[]) {
	if (argc < 2) {
		std::cerr << "Usage: rd <expr>" << std::endl;
		return -1;
	}

	std::string expr = argv[1];

	try {
		Interpreter p(expr);
		int n = p.interpret();
		std::cout << n << std::endl;
	}
	catch (InterpreterError &e) {
		std::cerr << "Interpreter error: " << e.message << " at position " << e.pos << std::endl;
		// Print the expression and caret below to visually indicate where exactly the parser error occurred
		std::cerr << expr << std::endl;
		for (size_t i=0; i<e.pos; i++) {
			std::cerr << " ";
		}
		std::cerr << "^" << std::endl;
		return -1;
	}

	return 0;
}
